@@ -1,14 +1,15 @@
 #!/usr/bin/ruby
-# = Ruby/ActiveLdap
+# = ActiveLdap
 #
-# "Ruby/ActiveLdap" Copyright (C) 2004,2005 Will Drewry mailto:will@alum.bu.edu
+# "ActiveLdap" Copyright (C) 2004,2005 Will Drewry mailto:will@alum.bu.edu
+#              Copyright (C) 2006-2009 Kouhei Sutou <kou@cozmixng.org>
 #
 # == Introduction
 #
-# Ruby/ActiveLdap is a novel way of interacting with LDAP.  Most interaction with
+# ActiveLdap is a novel way of interacting with LDAP.  Most interaction with
 # LDAP is done using clunky LDIFs, web interfaces, or with painful APIs that
-# required a thick reference manual nearby. Ruby/ActiveLdap aims to fix that.
-# Inspired by ActiveRecord[http://activerecord.rubyonrails.org], Ruby/ActiveLdap provides an
+# required a thick reference manual nearby. ActiveLdap aims to fix that.
+# Inspired by ActiveRecord[http://activerecord.rubyonrails.org], ActiveLdap provides an
 # object oriented interface to LDAP entries.
 #
 # The target audience is system administrators and LDAP users everywhere that
@@ -29,40 +30,38 @@
 # * RFC1777[http://www.faqs.org/rfcs/rfc1777.html] - Lightweight Directory Access Protocol
 # * OpenLDAP[http://www.openldap.org]
 #
-# === So why use Ruby/ActiveLdap?
+# === So why use ActiveLdap?
 #
 # Well if you like to fumble around in the dark, dank innards of LDAP, you can
 # quit reading now.  However, if you'd like a cleaner way to integrate LDAP in to
-# your existing code, hopefully that's why you'll want to use Ruby/ActiveLdap.
+# your existing code, hopefully that's why you'll want to use ActiveLdap.
 #
 # Using LDAP directly (even with the excellent Ruby/LDAP), leaves you bound to
 # the world of the predefined LDAP API.  While this API is important for many
 # reasons, having to extract code out of LDAP search blocks and create huge
 # arrays of LDAP.mod entries make code harder to read, less intuitive, and just
-# less fun to write.  Hopefully, Ruby/ActiveLdap will remedy all of these
+# less fun to write.  Hopefully, ActiveLdap will remedy all of these
 # problems!
 #
 # == Getting Started
 #
-# Ruby/ActiveLdap does have some overhead when you get started.  You must not
+# ActiveLdap does have some overhead when you get started.  You must not
 # only install the package and all of it's requirements, but you must also make
 # customizations that will let it work in your environment.
 #
 # === Requirements
 #
-# * Ruby[http://www.ruby-lang.org] 1.8.x
-# * Ruby/LDAP[http://ruby-ldap.sourceforge.net]
-# * (Optional) Ruby/LDAP+GSSAPI[http://caliban.org/files/redhat/RPMS/i386/ruby-ldap-0.8.2-4.i386.rpm]
-# * An LDAP server compatible with Ruby/LDAP: OpenLDAP[http://www.openldap.org], etc
+# * A Ruby implementation: Ruby[http://www.ruby-lang.org] 1.8.x, 1.9.1 or JRuby[http://jruby.codehaus.org/]
+# * A LDAP library: Ruby/LDAP[http://code.google.com/p/ruby-activeldap/wiki/RubyLDAP] (for Ruby), Net::LDAP[http://rubyforge.org/projects/net-ldap/] (for Ruby or JRuby) or JNDI (for JRuby)
+# * A LDAP server: OpenLDAP[http://www.openldap.org], etc
 #   - Your LDAP server must allow root_dse queries to allow for schema queries
-# * Examples also require: Ruby/Password[http://raa.ruby-lang.org/project/ruby-password/]
 #
 # === Installation
 #
 # Assuming all the requirements are installed, you can install by grabbing the latest tgz file from
 # the download site[http://rubyforge.org/frs/?group_id=381].
 #
-# The following steps will get the Ruby/ActiveLdap installed in no time!
+# The following steps will get the ActiveLdap installed in no time!
 #
 #   $ tar -xzvf ruby-activeldap-current.tgz
 #   $ cd ruby-activeldap-VERSION
@@ -90,7 +89,7 @@
 #
 # === Customizations
 #
-# Now that Ruby/ActiveLdap is installed and working, we still have a few more
+# Now that ActiveLdap is installed and working, we still have a few more
 # steps to make it useful for programming.
 #
 # Let's say that you are writing a Ruby program for managing user and group
@@ -116,7 +115,7 @@
 #
 # == Usage
 #
-# This section covers using Ruby/ActiveLdap from writing extension classes to
+# This section covers using ActiveLdap from writing extension classes to
 # writing applications that use them.
 #
 # Just to give a taste of what's to come, here is a quick example using irb:
@@ -126,7 +125,7 @@
 # Here's an extension class that maps to the LDAP Group objects:
 #
 #   irb> class Group < ActiveLdap::Base
-#   irb* ldap_mapping
+#   irb*   ldap_mapping
 #   irb* end
 #
 # Here is the Group class in use:
@@ -135,10 +134,7 @@
 #   => ["root", "daemon", "bin", "sys", "adm", "tty", ..., "develop"]
 #
 #   irb> group = Group.find("develop")
-#   => #<Group:0x..........>
-#
-#   irb> group.members.collect {|member| member.uid}
-#   => ["drewry"]
+#   => #<Group objectClass:<...> ...>
 #
 #   irb> group.cn
 #   => "develop"
@@ -155,7 +151,7 @@
 #
 # ==== Why do I need them?
 #
-# Extension classes are what make Ruby/ActiveLdap "active"! They do all the
+# Extension classes are what make ActiveLdap "active"! They do all the
 # background work to make easy-to-use objects by mapping the LDAP object's
 # attributes on to a Ruby class.
 #
@@ -169,7 +165,7 @@
 # ===== ldap_mapping
 #
 # ldap_mapping is the only required method to setup an extension class for use
-# with Ruby/ActiveLdap. It must be called inside of a subclass as shown above.
+# with ActiveLdap. It must be called inside of a subclass as shown above.
 #
 # Below is a much more realistic Group class:
 #
@@ -208,9 +204,9 @@
 # for dn_attribute matches.
 # (e.g. cn=develop,ou=DevGroups,ou=Groups,dc=dataspill,dc=org)
 #
-# Something's missing: :classes.  :classes is used to tell Ruby/ActiveLdap what
+# Something's missing: :classes.  :classes is used to tell ActiveLdap what
 # the minimum requirement is when creating a new object. LDAP uses objectClasses
-# to define what attributes a LDAP object may have. Ruby/ActiveLdap needs to know
+# to define what attributes a LDAP object may have. ActiveLdap needs to know
 # what classes are required when creating a new object.  Of course, you can leave
 # that field out to default to ['top'] only.  Then you can let each application
 # choose what objectClasses their objects should have by calling the method e.g.
@@ -222,8 +218,8 @@
 # may help avoid programmer error later.
 #
 # :classes isn't the only optional argument.  If :dn_attribute is left off,
-# it defaults to underscored class name or 'cn'.  If :prefix is left off,
-# it will default to 'ou=PLURALIZED_CLASSNAME'. In this
+# it defaults to super class's value or 'cn'.  If :prefix is left off,
+# it will default to 'ou=PluralizedClassName'. In this
 # case, it would be 'ou=Groups'.
 #
 # :classes should be an Array. :dn_attribute should be a String and so should
@@ -256,7 +252,7 @@
 #
 #   irb> class User < ActiveLdap::Base
 #   irb*   ldap_mapping :dn_attribute => 'uid', :prefix => 'People', :classes => ['top','account']
-#   irb*   belongs_to :groups, :class => 'Group', :many => 'memberUid', :foreign_key => 'uid'
+#   irb*   belongs_to :groups, :class_name => 'Group', :many => 'memberUid', :foreign_key => 'uid'
 #   irb* end
 #
 # Now, class User will have a method called 'groups' which will retrieve all
@@ -264,7 +260,7 @@
 #
 #   irb> me = User.find('drewry')
 #   irb> me.groups
-#   => [#<Group:0x000001 ...>, #<Group:0x000002 ...>, ...]
+#   => [#<Group ...>, #<Group ...>, ...]
 #   irb> me.groups.each { |group| p group.cn };nil
 #   "cdrom"
 #   "audio"
@@ -284,18 +280,18 @@
 # Now let's talk about the arguments.  The first argument is the name of the
 # method you wish to create. In this case, we created a method called groups
 # using the symbol :groups. The next collection of arguments are actually a Hash
-# (as with ldap_mapping). :class should be a string that has the name of a
+# (as with ldap_mapping). :class_name should be a string that has the name of a
 # class you've already included. If you class is inside of a module, be sure to
-# put the whole name, e.g. :class => "MyLdapModule::Group". :primary_key
+# put the whole name, e.g. :class_name => "MyLdapModule::Group". :foreign_key
 # tells belongs_to what attribute Group objects have that match the
 # :many. :many is the name of the local attribute whose value
-# should be looked up in Group under the primary key. If :foreign_key is left
+# should be looked up in Group under the foreign key. If :foreign_key is left
 # off of the argument list, it is assumed to be the dn_attribute. With this in
 # mind, the above definition could become:
 #
 #   irb> class User < ActiveLdap::Base
 #   irb*   ldap_mapping :dn_attribute => 'uid', :prefix => 'People', :classes => ['top','account']
-#   irb*   belongs_to :groups, :class => 'Group', :many => 'memberUid'
+#   irb*   belongs_to :groups, :class_name => 'Group', :many => 'memberUid'
 #   irb* end
 #
 # In addition, you can do simple membership tests by doing the following:
@@ -314,7 +310,7 @@
 #
 #   class Group < ActiveLdap::Base
 #     ldap_mapping :dn_attribute => 'cn', :prefix => 'ou=Groups', :classes => ['top', 'posixGroup']
-#     has_many :members, :class => "User", :wrap => "memberUid", :primary_key => 'uid'
+#     has_many :members, :class_name => "User", :wrap => "memberUid", :primary_key => 'uid'
 #   end
 #
 # Now we can see that group develop has user 'drewry' as a member, and it can
@@ -323,13 +319,13 @@
 #   irb> develop = Group.find('develop')
 #   => ...
 #   irb> develop.members
-#   => [#<User:0x000001 ...>, #<User:...>]
+#   => [#<User ...>, #<User ...>]
 #
 #
 # The arguments for has_many follow the exact same idea that belongs_to's
 # arguments followed. :wrap's contents are used to search for matching
 # :primary_key content.  If :primary_key is not specified, it defaults to the
-# dn_attribute of the specified :class.
+# dn_attribute of the specified :class_name.
 #
 # === Using these new classes
 #
@@ -341,21 +337,21 @@
 #
 # ==== .find
 #
-# .find is a class method that is accessible from any subclass of Base that has
-# 'ldap_mapping' called. When called it returns the first match of the given
-# class.
+# .find is a class method that is accessible from
+# any subclass of Base that has 'ldap_mapping' called. When
+# called .first(:first) returns the first match of the given class.
 #
-#   irb> Group.find('*').cn
-#   => "root"
+#   irb> Group.find(:first, 'deve*").cn
+#   => "develop"
 #
 # In this simple example, Group.find took the search string of 'deve*' and
 # searched for the first match in Group where the dn_attribute matched the
 # query. This is the simplest example of .find.
 #
-#   irb> Group.find(:all, '*').collect {|group| group.cn}
+#   irb> Group.find(:all).collect {|group| group.cn}
 #   => ["root", "daemon", "bin", "sys", "adm", "tty", ..., "develop"]
 #
-# Here .find(:all) returns all matches to the same query.  Both .find and
+# Here .find(:all) returns all matches to the same query.  Both .find(:first) and
 # .find(:all) also can take more expressive arguments:
 #
 #   irb> Group.find(:all, :attribute => 'gidNumber', :value => '1003').collect {|group| group.cn}
@@ -412,25 +408,25 @@
 #
 # ==== What is it?
 #
-# ActiveLdap::Base is the heart of Ruby/ActiveLdap.  It does all the schema
+# ActiveLdap::Base is the heart of ActiveLdap.  It does all the schema
 # parsing for validation and attribute-to-method mangling as well as manage the
 # connection to LDAP.
 #
-# ===== establish_connection
+# ===== setup_connection
 #
-# Base.establish_connection takes many (optional) arguments and is used to
+# Base.setup_connection takes many (optional) arguments and is used to
 # connect to the LDAP server. Sometimes you will want to connect anonymously
-# and other times over TLS with user credentials. Base.establish_connection is
+# and other times over TLS with user credentials. Base.setup_connection is
 # here to do all of that for you.
 #
 #
 # By default, if you call any subclass of Base, such as Group, it will call
-# Base.establish_connection() if these is no active LDAP connection. If your
+# Base.setup_connection() if these is no active LDAP connection. If your
 # server allows anonymous binding, and you only want to access data in a
-# read-only fashion, you won't need to call Base.establish_connection. Here
+# read-only fashion, you won't need to call Base.setup_connection. Here
 # is a fully parameterized call:
 #
-#   Base.establish_connection(
+#   Base.setup_connection(
 #     :host => 'ldap.dataspill.org',
 #     :port => 389,
 #     :base => 'dc=dataspill,dc=org',
@@ -446,7 +442,7 @@
 # * :port defaults to @@port from configuration.rb as well
 # * :base defaults to Base.base() from configuration.rb
 # * :bind_dn defaults @@bind_format from configuration.rb
-# * :logger defaults to a Log4r object that prints fatal messages to stderr
+# * :logger defaults to a Logger object that prints fatal messages to stderr
 # * :password_block defaults to nil
 # * :allow_anonymous defaults to true
 # * :try_sasl defaults to false - see Advanced Topics for more on this one.
@@ -471,15 +467,16 @@
 # * :try_sasl, when true, tells ActiveLdap to attempt a SASL-GSSAPI bind
 # * :sasl_quiet, when true, tells the SASL libraries to not spew messages to STDOUT
 # * :method indicates whether to use :ssl, :tls, or :plain
-# * :retries - indicates the number of attempts to reconnect that will be undertaken when a stale connection occurs. -1 means infinite.
+# * :retry_limit - indicates the number of attempts to reconnect that will be undertaken when a stale connection occurs. -1 means infinite.
 # * :retry_wait - seconds to wait before retrying a connection
 # * :scope - dictates how to find objects. (Default: :one)
 # * :timeout - time in seconds - defaults to disabled. This CAN interrupt search() requests. Be warned.
 # * :retry_on_timeout - whether to reconnect when timeouts occur. Defaults to true
 # See lib/configuration.rb for defaults for each option
 #
-# Base.establish_connection both connects and binds in one step. It follows
-# roughly the following approach:
+# Base.setup_connection just setups connection
+# configuration. A connection is connected and bound when it
+# is needed. It follows roughly the following approach:
 #
 # * Connect to host:port using :method
 #
@@ -488,9 +485,10 @@
 #   anonymously.
 # * If that fails, error out.
 #
-# On connect, the configuration options passed in are stored in an internal class variable
-# @configuration which is used to cache the information without ditching the defaults passed in
-# from configuration.rb
+# On connect, the configuration options passed in are stored
+# in an internal class variable which is used to cache the
+# information without ditching the defaults passed in from
+# configuration.rb
 #
 # ===== connection
 #
@@ -498,7 +496,7 @@
 #
 # === Exceptions
 #
-# There are a few custom exceptions used in Ruby/ActiveLdap. They are detailed below.
+# There are a few custom exceptions used in ActiveLdap. They are detailed below.
 #
 # ==== DeleteError
 #
@@ -514,14 +512,14 @@
 #
 # ==== AuthenticationError
 #
-# This exception is raised during Base.establish_connection if no valid authentication methods
+# This exception is raised during Base.setup_connection if no valid authentication methods
 # succeeded.
 #
 # ==== ConnectionError
 #
-# This exception is raised during Base.establish_connection if no valid
+# This exception is raised during Base.setup_connection if no valid
 # connection to the LDAP server could be created. Check you configuration.rb,
-# Base.establish_connection arguments, and network connectivity! Also check
+# Base.setup_connection arguments, and network connectivity! Also check
 # your LDAP server logs to see if it ever saw the request.
 #
 # ==== ObjectClassError
@@ -538,7 +536,7 @@
 #
 # === Putting it all together
 #
-# Now that all of the components of Ruby/ActiveLdap have been covered, it's time
+# Now that all of the components of ActiveLdap have been covered, it's time
 # to put it all together! The rest of this section will show the steps to setup
 # example user and group management scripts for use with the LDAP tree described
 # above.
@@ -551,7 +549,7 @@
 #   cat <<EOF
 #   class User < ActiveLdap::Base
 #     ldap_mapping :dn_attribute => 'uid', :prefix => 'ou=People', :classes => ['top', 'account', 'posixAccount']
-#     belongs_to :groups, :class => 'Group', :wrap => 'memberUid'
+#     belongs_to :groups, :class_name => 'Group', :wrap => 'memberUid'
 #   end
 #   EOF
 #
@@ -559,8 +557,8 @@
 #   cat <<EOF
 #   class Group < ActiveLdap::Base
 #     ldap_mapping :classes => ['top', 'posixGroup'], :prefix => 'ou=Group'
-#     has_many :members, :class => "User", :many => "memberUid"
-#     has_many :primary_members, :class => 'User', :foreign_key => 'gidNumber', :primary_key => 'gidNumber'
+#     has_many :members, :class_name => "User", :many => "memberUid"
+#     has_many :primary_members, :class_name => 'User', :foreign_key => 'gidNumber', :primary_key => 'gidNumber'
 #   end # Group
 #   EOF
 #
@@ -570,36 +568,37 @@
 #
 # Now let's create a really dumb script for adding users - ldapadmin/useradd:
 #
-#   #!/usr/bin/ruby -W0
-#
+#   base = File.expand_path(File.join(File.dirname(__FILE__), ".."))
+#   $LOAD_PATH << File.join(base, "lib")
+#   $LOAD_PATH << File.join(base, "examples")
+#   
 #   require 'active_ldap'
-#   require 'lib/user'
-#   require 'lib/group'
-#   require 'password'
-#
+#   require 'objects/user'
+#   require 'objects/group'
+#   
 #   argv, opts, options = ActiveLdap::Command.parse_options do |opts, options|
 #     opts.banner += " USER_NAME CN UID"
 #   end
-#
+#   
 #   if argv.size == 3
 #     name, cn, uid = argv
 #   else
 #     $stderr.puts opts
 #     exit 1
 #   end
-#
+#   
 #   pwb = Proc.new do |user|
 #     ActiveLdap::Command.read_password("[#{user}] Password: ")
 #   end
-#
-#   ActiveLdap::Base.establish_connection(:password_block => pwb,
-#                                         :allow_anonymous => false)
-#
+#   
+#   ActiveLdap::Base.setup_connection(:password_block => pwb,
+#                                     :allow_anonymous => false)
+#   
 #   if User.exists?(name)
 #     $stderr.puts("User #{name} already exists.")
 #     exit 1
 #   end
-#
+#   
 #   user = User.new(name)
 #   user.add_class('shadowAccount')
 #   user.cn = cn
@@ -618,34 +617,38 @@
 # Now let's create another dumb script for modifying users - ldapadmin/usermod:
 #
 #   #!/usr/bin/ruby -W0
-#
+#   
+#   base = File.expand_path(File.join(File.dirname(__FILE__), ".."))
+#   $LOAD_PATH << File.join(base, "lib")
+#   $LOAD_PATH << File.join(base, "examples")
+#   
 #   require 'active_ldap'
-#   require 'lib/user'
-#   require 'lib/group'
-#
+#   require 'objects/user'
+#   require 'objects/group'
+#   
 #   argv, opts, options = ActiveLdap::Command.parse_options do |opts, options|
 #     opts.banner += " USER_NAME CN UID"
 #   end
-#
+#   
 #   if argv.size == 3
 #     name, cn, uid = argv
 #   else
 #     $stderr.puts opts
 #     exit 1
 #   end
-#
+#   
 #   pwb = Proc.new do |user|
 #     ActiveLdap::Command.read_password("[#{user}] Password: ")
 #   end
-#
-#   ActiveLdap::Base.establish_connection(:password_block => pwb,
-#                                         :allow_anonymous => false)
-#
+#   
+#   ActiveLdap::Base.setup_connection(:password_block => pwb,
+#                                     :allow_anonymous => false)
+#   
 #   unless User.exists?(name)
 #     $stderr.puts("User #{name} doesn't exist.")
 #     exit 1
 #   end
-#
+#   
 #   user = User.find(name)
 #   user.cn = cn
 #   user.uid_number = uid
@@ -656,45 +659,9 @@
 #     exit 1
 #   end
 #
-# ==== Removing LDAP entries
-#
-# And finally, a dumb script for removing user - ldapadmin/userdel:
-#
-#
-#   #!/usr/bin/ruby -W0
-#
-#   require 'active_ldap'
-#   require 'lib/user'
-#   require 'lib/group'
-#
-#   argv, opts, options = ActiveLdap::Command.parse_options do |opts, options|
-#     opts.banner += " USER_NAME"
-#   end
-#
-#   if argv.size == 1
-#     name = argv.shift
-#   else
-#     $stderr.puts opts
-#     exit 1
-#   end
-#
-#   pwb = Proc.new do |user|
-#     ActiveLdap::Command.read_password("[#{user}] Password: ")
-#   end
-#
-#   ActiveLdap::Base.establish_connection(:password_block => pwb,
-#                                         :allow_anonymous => false)
-#
-#   unless User.exists?(name)
-#     $stderr.puts("User #{name} doesn't exist.")
-#     exit 1
-#   end
-#
-#   User.destroy(name)
-#
 # === Advanced Topics
 #
-# Below are some situation tips and tricks to get the most out of Ruby/ActiveLdap.
+# Below are some situation tips and tricks to get the most out of ActiveLdap.
 #
 #
 # ==== Binary data and other subtypes
@@ -706,7 +673,7 @@
 #   irb> user = User.new('drewry')
 #   => ...
 #   # This adds a cn entry in lang-en and whatever the server default is.
-#   irb> user.cn = [ 'wad', {'lang-en' => ['wad', 'foo']} ]
+#   irb> user.cn = [ 'wad', {'lang-en' => ['wad', 'Will Drewry']} ]
 #   => ...
 #   irb> user.cn
 #   => ["wad", {"lang-en-us" => ["wad", "Will Drewry"]}]
@@ -729,7 +696,7 @@
 #
 # You should note that some binary data does not use the binary subtype all the time.
 # One example is jpegPhoto. You can use it as jpegPhoto;binary or just as jpegPhoto.
-# Since the schema dictates that it is a binary value, Ruby/ActiveLdap will write
+# Since the schema dictates that it is a binary value, ActiveLdap will write
 # it as binary, but the subtype will not be automatically appended as above. The
 # use of the subtype on attributes like jpegPhoto is ultimately decided by the
 # LDAP site policy and not by any programmatic means.
@@ -737,7 +704,7 @@
 # The only subtypes defined in LDAPv3 are lang-* and binary.  These can be nested
 # though:
 #
-#  irb> user.cn = [{'lang-JP-jp' => {'binary' => 'somejp'}}]
+#  irb> user.cn = [{'lang-ja' => {'binary' => 'some Japanese'}}]
 #
 # As I understand it, OpenLDAP does not support nested subtypes, but some
 # documentation I've read suggests that Netscape's LDAP server does. I only
@@ -766,7 +733,7 @@
 #   module MyLDAP
 #   class User < ActiveLdap::Base
 #     ldap_mapping :dn_attribute => 'uid', :prefix => 'ou=People', :classes => ['top', 'account', 'posixAccount']
-#     belongs_to :groups, :class => 'MyLDAP::Group', :many => 'memberUid'
+#     belongs_to :groups, :class_name => 'MyLDAP::Group', :many => 'memberUid'
 #   end
 #   end
 #
@@ -774,8 +741,8 @@
 #   module MyLDAP
 #   class Group < ActiveLdap::Base
 #     ldap_mapping :classes => ['top', 'posixGroup'], :prefix => 'ou=Group'
-#     has_many :members, :class => 'MyLDAP::User', :wrap => 'memberUid'
-#     has_many :primary_members, :class => 'MyLDAP::User', :foreign_key => 'gidNumber', :primary_key => 'gidNumber'
+#     has_many :members, :class_name => 'MyLDAP::User', :wrap => 'memberUid'
+#     has_many :primary_members, :class_name => 'MyLDAP::User', :foreign_key => 'gidNumber', :primary_key => 'gidNumber'
 #   end
 #   end
 #
@@ -791,7 +758,7 @@
 #
 # ==== force array results for single values
 #
-# Even though Ruby/ActiveLdap attempts to maintain programmatic ease by
+# Even though ActiveLdap attempts to maintain programmatic ease by
 # returning Array values only. By specifying 'true' as an argument to
 # any attribute method you will get back a Array if it is single value.
 # Here's an example:
@@ -820,11 +787,11 @@
 #
 #   irb> anon_class = Class.new(Base)
 #   => ...
-#   irb> anon_class.establish_connection
+#   irb> anon_class.setup_connection
 #   => ...
 #   irb> auth_class = Class.new(Base)
 #   => ...
-#   irb> auth_class.establish_connection(:password_block => {'mypass'})
+#   irb> auth_class.setup_connection(:password_block => {'mypass'})
 #   => ...
 #
 # This can be useful for doing authentication tests and other such tricks.
@@ -893,7 +860,7 @@
 #
 # === Speed
 #
-# Currently, Ruby/ActiveLdap could be faster.  I have some recursive type
+# Currently, ActiveLdap could be faster.  I have some recursive type
 # checking going on which slows object creation down, and I'm sure there
 # are many, many other places optimizations can be done.  Feel free
 # to send patches, or just hang in there until I can optimize away the
@@ -905,12 +872,16 @@
 # package, and I'd like to see it prove helpful to more people than just myself.
 #
 
-require_gem_if_need = Proc.new do |library_name, gem_name|
+require_gem_if_need = Proc.new do |library_name, gem_name, *gem_args|
+  gem_name ||= library_name
   begin
+    if !gem_args.empty? and Object.const_defined?(:Gem)
+      gem gem_name, *gem_args
+    end
     require library_name
   rescue LoadError
     require 'rubygems'
-    gem gem_name
+    gem gem_name, *gem_args
     require library_name
   end
 end
@@ -928,7 +899,7 @@ if dependencies.respond_to?(:load_paths)
 end
 
 module ActiveLdap
-  VERSION = "1.0.1"
+  VERSION = "1.0.9"
 end
 
 if RUBY_PLATFORM.match('linux')
@@ -939,15 +910,30 @@ end
 
 require_gem_if_need.call("active_record", "activerecord")
 begin
-  require_gem_if_need.call("gettext/active_record", "gettext")
+  raise LoadError, "GetText is disabled."
+  require_gem_if_need.call("locale")
+  require_gem_if_need.call("gettext")
+
+  require 'active_record/version'
+  active_record_version = [ActiveRecord::VERSION::MAJOR,
+                           ActiveRecord::VERSION::MINOR,
+                           ActiveRecord::VERSION::TINY]
+  if (active_record_version <=> [2, 2, 0]) < 0
+    require "gettext/active_record"
+  else
+    require_gem_if_need.call("gettext_activerecord")
+  end
 rescue LoadError
 end
 require 'active_ldap/get_text'
 
+require 'active_ldap/compatible'
+
 require 'active_ldap/base'
 
 require 'active_ldap/distinguished_name'
 require 'active_ldap/ldif'
+require 'active_ldap/xml'
 
 require 'active_ldap/associations'
 require 'active_ldap/attributes'
@@ -961,6 +947,7 @@ require 'active_ldap/acts/tree'
 
 require 'active_ldap/populate'
 require 'active_ldap/escape'
+require 'active_ldap/user_password'
 require 'active_ldap/helper'
 
 require 'active_ldap/validations'
