@@ -24,6 +24,8 @@ module ActiveLdap
         @runtime = 0
         @connection = nil
         @disconnected = false
+        @bound = false
+        @bind_tried = false
         @entry_attributes = {}
         @configuration = configuration.dup
         @logger = @configuration.delete(:logger)
@@ -40,18 +42,21 @@ module ActiveLdap
 
       def connect(options={})
         host = options[:host] || @host
-        port = options[:port] || @port
-        method = ensure_method(options[:method] || @method)
+        method = options[:method] || @method || :plain
+        port = options[:port] || @port || ensure_port(method)
+        method = ensure_method(method)
         @disconnected = false
+        @bound = false
+        @bind_tried = false
         @connection, @uri, @with_start_tls = yield(host, port, method)
         prepare_connection(options)
         bind(options)
       end
 
       def disconnect!(options={})
-        return if @connection.nil?
         unbind(options)
         @connection = @uri = @with_start_tls = nil
+        @disconnected = true
       end
 
       def rebind(options={})
@@ -60,6 +65,8 @@ module ActiveLdap
       end
 
       def bind(options={})
+        @bind_tried = true
+
         bind_dn = options[:bind_dn] || @bind_dn
         try_sasl = options.has_key?(:try_sasl) ? options[:try_sasl] : @try_sasl
         if options.has_key?(:allow_anonymous)
@@ -85,19 +92,27 @@ module ActiveLdap
           raise AuthenticationError, message
         end
 
-        bound?
+        @bound = true
+        @bound
+      end
+
+      def unbind(options={})
+        yield if @connection and (@bind_tried or bound?)
+        @bind_tried = @bound = false
       end
 
       def bind_as_anonymous(options={})
-        operation(options) do
-          yield
-        end
+        yield
       end
 
       def connecting?
         !@connection.nil? and !@disconnected
       end
 
+      def bound?
+        connecting? and @bound
+      end
+
       def schema(options={})
         @schema ||= operation(options) do
           base = options[:base]
@@ -163,11 +178,14 @@ module ActiveLdap
       def delete(targets, options={})
         targets = [targets] unless targets.is_a?(Array)
         return if targets.empty?
-        target = nil
         begin
           operation(options) do
             targets.each do |target|
-              yield(target)
+              begin
+                yield(target)
+              rescue LdapError::UnwillingToPerform, LdapError::InsufficientAccess
+                raise OperationNotPermitted, _("%s: %s") % [$!.message, target]
+              end
             end
           end
         rescue LdapError::NoSuchObject
@@ -198,7 +216,11 @@ module ActiveLdap
       def modify(dn, entries, options={})
         begin
           operation(options) do
-            yield(dn, entries)
+            begin
+              yield(dn, entries)
+            rescue LdapError::UnwillingToPerform, LdapError::InsufficientAccess
+              raise OperationNotPermitted, _("%s: %s") % [$!.message, target]
+            end
           end
         rescue LdapError::UndefinedType
           raise
@@ -213,14 +235,22 @@ module ActiveLdap
         end
       end
 
-      def log_info(name, runtime, info=nil)
+      def log_info(name, runtime_in_seconds, info=nil)
         return unless @logger
         return unless @logger.debug?
-        message = "LDAP: #{name} (#{'%f' % runtime})"
+        message = "LDAP: #{name} (#{'%.1f' % (runtime_in_seconds * 1000)}ms)"
         @logger.debug(format_log_entry(message, info))
       end
 
       private
+      def ensure_port(method)
+        if method == :ssl
+          URI::LDAPS::DEFAULT_PORT
+        else
+          URI::LDAP::DEFAULT_PORT
+        end
+      end
+
       def prepare_connection(options)
       end
 
@@ -276,11 +306,19 @@ module ActiveLdap
       end
 
       def with_timeout(try_reconnect=true, options={}, &block)
+        n_retries = 0
+        retry_limit = options[:retry_limit] || @retry_limit
         begin
           Timeout.alarm(@timeout, &block)
         rescue Timeout::Error => e
           @logger.error {_('Requested action timed out.')}
-          retry if @retry_on_timeout and try_reconnect and reconnect(options)
+          if @retry_on_timeout and retry_limit < 0 and n_retries <= retry_limit
+            if connecting?
+              retry
+            elsif try_reconnect
+              retry if with_timeout(false, options) {reconnect(options)}
+            end
+          end
           @logger.error {e.message}
           raise TimeoutError, e.message
         end
@@ -301,10 +339,7 @@ module ActiveLdap
         sasl_mechanisms = options[:sasl_mechanisms] || @sasl_mechanisms
         sasl_mechanisms.each do |mechanism|
           next unless mechanisms.include?(mechanism)
-          operation(options) do
-            yield(bind_dn, mechanism, sasl_quiet)
-            return true if bound?
-          end
+          return true if yield(bind_dn, mechanism, sasl_quiet)
         end
         false
       end
@@ -326,10 +361,7 @@ module ActiveLdap
         end
 
         begin
-          operation(options) do
-            yield(bind_dn, passwd)
-            bound?
-          end
+          yield(bind_dn, passwd)
         rescue LdapError::InvalidDnSyntax
           raise DistinguishedNameInvalid.new(bind_dn)
         rescue LdapError::InvalidCredentials
@@ -388,7 +420,7 @@ module ActiveLdap
           when Hash
             options = value[0]
             value = value[1]
-          when "=", "~=", "<=", "=>"
+          when "=", "~=", "<=", ">="
             options[:operator] = value[0]
             if value.size > 2
               value = value[1..-1]
@@ -467,7 +499,11 @@ module ActiveLdap
             s
           else
             s = "*" if s == "**"
-            "\\%02X" % s[0]
+            if s.respond_to?(:getbyte)
+              "\\%02X" % s.getbyte(0)
+            else
+              "\\%02X" % s[0]
+            end
           end
         end
       end
@@ -527,11 +563,6 @@ module ActiveLdap
         options[:reconnect_attempts] ||= 0
 
         loop do
-          unless can_reconnect?(options)
-            raise ConnectionError,
-                  _('Giving up trying to reconnect to LDAP server.')
-          end
-
           @logger.debug {_('Attempting to reconnect')}
           disconnect!
 
@@ -541,6 +572,8 @@ module ActiveLdap
           begin
             connect(options)
             break
+          rescue AuthenticationError
+            raise
           rescue => detail
             @logger.error do
               _("Reconnect to server failed: %s\n" \
@@ -551,6 +584,11 @@ module ActiveLdap
             raise ConnectionError, detail.message if force
           end
 
+          unless can_reconnect?(options)
+            raise ConnectionError,
+                  _('Giving up trying to reconnect to LDAP server.')
+          end
+
           # Sleep before looping
           sleep retry_wait
         end
@@ -559,7 +597,10 @@ module ActiveLdap
       end
 
       def reconnect_if_need(options={})
-        reconnect(options) if !connecting? and can_reconnect?(options)
+        return if connecting?
+        with_timeout(false, options) do
+          reconnect(options)
+        end
       end
 
       # Determine if we have exceed the retry limit or not.
@@ -568,7 +609,7 @@ module ActiveLdap
         retry_limit = options[:retry_limit] || @retry_limit
         reconnect_attempts = options[:reconnect_attempts] || 0
 
-        retry_limit < 0 or reconnect_attempts < (retry_limit - 1)
+        retry_limit < 0 or reconnect_attempts <= retry_limit
       end
 
       def root_dse_values(key, options={})
@@ -601,17 +642,13 @@ module ActiveLdap
 
       def log(name, info=nil)
         if block_given?
-          if @logger and @logger.debug?
-            result = nil
-            runtime = Benchmark.realtime {result = yield}
-            @runtime += runtime
-            log_info(name, runtime, info)
-            result
-          else
-            yield
-          end
+          result = nil
+          seconds = Benchmark.realtime {result = yield}
+          @runtime += seconds
+          log_info(name, seconds, info)
+          result
         else
-          log_info(name, info, 0)
+          log_info(name, 0, info)
           nil
         end
       rescue Exception
