@@ -30,6 +30,7 @@
 
 require 'English'
 require 'thread'
+require 'erb'
 
 module ActiveLdap
   # OO-interface to LDAP assuming pam/nss_ldap-style organization with
@@ -43,6 +44,24 @@ module ActiveLdap
   # Multiple entries become lists.
   # If this isn't read-only then lists become multiple entries, etc.
 
+  class << self
+    include GetTextSupport
+    def const_missing(id)
+      case id
+      when :ConnectionNotEstablished
+        message =
+          _("ActiveLdap::ConnectionNotEstablished has been deprecated " \
+            "since 1.1.0. " \
+            "Please use ActiveLdap::ConnectionNotSetup instead.")
+        ActiveSupport::Deprecation.warn(message)
+        const_set("ConnectionNotEstablished", ConnectionNotSetup)
+        ConnectionNotEstablished
+      else
+        super
+      end
+    end
+  end
+
   class Error < StandardError
     include GetTextSupport
   end
@@ -154,13 +173,16 @@ module ActiveLdap
     NEAREST_MARK = "|@|"
     private
     def detect_nearest(line, column)
-      nearest = @ldif.to_a[line - 1] || ""
+      lines = Compatible.string_to_lines(@ldif).to_a
+      nearest = lines[line - 1] || ""
       if column - 1 == nearest.size # for JRuby 1.0.2 :<
         nearest << NEAREST_MARK
       else
         nearest[column - 1, 0] = NEAREST_MARK
       end
-      nearest = "#{@ldif.to_a[line - 2]}#{nearest}" if nearest == NEAREST_MARK
+      if nearest == NEAREST_MARK
+        nearest = "#{lines[line - 2]}#{nearest}"
+      end
       nearest
     end
 
@@ -181,7 +203,7 @@ module ActiveLdap
 
     def numbered_ldif
       return @ldif if @ldif.blank?
-      lines = @ldif.to_a
+      lines = Compatible.string_to_lines(@ldif)
       format = "%#{Math.log10(lines.size).truncate + 1}d: %s"
       i = 0
       lines.collect do |line|
@@ -206,7 +228,7 @@ module ActiveLdap
   class OperationNotPermitted < Error
   end
 
-  class ConnectionNotEstablished < Error
+  class ConnectionNotSetup < Error
   end
 
   class AdapterNotSpecified < Error
@@ -287,7 +309,6 @@ module ActiveLdap
           end
           def #{sym}; self.class.#{sym}; end
           def self.#{sym}=(value); @#{sym} = value; end
-          def #{sym}=(value); self.class.#{sym} = value; end
         EOS
       end
     end
@@ -339,13 +360,24 @@ module ActiveLdap
       #   search() requests. Be warned.
       # :retry_on_timeout - whether to reconnect when timeouts occur. Defaults
       #   to true
-      # See lib/configuration.rb for defaults for each option
-      def establish_connection(config=nil)
+      # See lib/active_ldap/configuration.rb for defaults for each option
+      def setup_connection(config=nil)
         super
         ensure_logger
         nil
       end
 
+      # establish_connection is deprecated since 1.1.0. Please use
+      # setup_connection() instead.
+      def establish_connection(config=nil)
+        message =
+          _("ActiveLdap::Base.establish_connection has been deprecated " \
+            "since 1.1.0. " \
+            "Please use ActiveLdap::Base.setup_connection instead.")
+        ActiveSupport::Deprecation.warn(message)
+        setup_connection(config)
+      end
+
       def create(attributes=nil, &block)
         if attributes.is_a?(Array)
           attributes.collect {|attrs| create(attrs, &block)}
@@ -438,6 +470,8 @@ module ActiveLdap
       def inspect
         if self == Base
           super
+        elsif abstract_class?
+          "#{super}(abstract)"
         else
           class_names = []
           must = []
@@ -454,6 +488,54 @@ module ActiveLdap
         end
       end
 
+      attr_accessor :abstract_class
+      def abstract_class?
+        defined?(@abstract_class) && @abstract_class
+      end
+
+      def class_of_active_ldap_descendant(klass)
+        if klass.superclass == Base or klass.superclass.abstract_class?
+          klass
+        elsif klass.superclass.nil?
+          raise Error, _("%s doesn't belong in a hierarchy descending " \
+                         "from ActiveLdap") % (name || to_s)
+        else
+          class_of_active_ldap_descendant(klass.superclass)
+        end
+      end
+
+      def self_and_descendants_from_active_ldap
+        klass = self
+        classes = [klass]
+        while klass != klass.base_class
+          classes << klass = klass.superclass
+        end
+        classes
+      rescue
+        [self]
+      end
+      if ActiveRecord::Base.respond_to?(:self_and_descendents_from_active_record)
+        # ActiveRecord 2.2.2 has a typo. :<
+        alias_method(:self_and_descendents_from_active_record,
+                     :self_and_descendants_from_active_ldap)
+      else
+        alias_method(:self_and_descendants_from_active_record,
+                     :self_and_descendants_from_active_ldap)
+      end
+
+      def human_name(options={})
+        defaults = self_and_descendants_from_active_ldap.collect do |klass|
+          if klass.name.blank?
+            nil
+          else
+            :"#{klass.name.underscore}"
+          end
+        end
+        defaults << name.humanize
+        defaults = defaults.compact
+        defaults.first || name || to_s
+      end
+
       private
       def inspect_attributes(attributes)
 	inspected_attribute_names = {}
@@ -518,20 +600,16 @@ module ActiveLdap
       end
 
       def default_dn_attribute
-        if name.empty?
-          dn_attribute = nil
-          parent_class = ancestors[1]
-          if parent_class.respond_to?(:dn_attribute)
-            dn_attribute = parent_class.dn_attribute
-          end
-          dn_attribute || "cn"
-        else
-          name.demodulize.underscore
+        dn_attribute = nil
+        parent_class = ancestors[1]
+        if parent_class.respond_to?(:dn_attribute)
+          dn_attribute = parent_class.dn_attribute
         end
+        dn_attribute || "cn"
       end
 
       def default_prefix
-        if name.empty?
+        if name.blank?
           nil
         else
           "ou=#{name.demodulize.pluralize}"
@@ -579,7 +657,6 @@ module ActiveLdap
         raise ArgumentError, format % attributes.inspect
       end
       yield self if block_given?
-      assert_dn_attribute
     end
 
     # Returns true if the +comparison_object+ is the same object, or is of
@@ -622,7 +699,7 @@ module ActiveLdap
 
     def attribute_present?(name)
       values = get_attribute(name, true)
-      !values.empty? or values.any? {|x| not (x and x.empty?)}
+      !values.empty? or values.any? {|x| !(x and x.empty?)}
     end
 
     # exist?
@@ -648,7 +725,7 @@ module ActiveLdap
     end
 
     def id
-      get_attribute(dn_attribute)
+      get_attribute(dn_attribute_with_fallback)
     end
 
     def to_param
@@ -656,13 +733,14 @@ module ActiveLdap
     end
 
     def dn=(value)
-      set_attribute(dn_attribute, value)
+      set_attribute(dn_attribute_with_fallback, value)
       @dn = nil
     end
     alias_method(:id=, :dn=)
 
     alias_method(:dn_attribute_of_class, :dn_attribute)
     def dn_attribute
+      ensure_update_dn
       _dn_attribute = @dn_attribute || dn_attribute_of_class
       to_real_attribute_name(_dn_attribute) || _dn_attribute
     end
@@ -792,7 +870,7 @@ module ActiveLdap
     # Also be sure to only pass in key-value pairs of your choosing.
     # Do not let URL/form hackers supply the keys.
     def attributes=(new_attributes)
-      return if new_attributes.nil?
+      return if new_attributes.blank?
       _schema = _local_entry_attribute = nil
       targets = remove_attributes_protected_from_mass_assignment(new_attributes)
       targets.each do |key, value|
@@ -817,31 +895,28 @@ module ActiveLdap
     end
 
     def to_xml(options={})
-      root = options[:root] || self.class.name.underscore
-      result = "<#{root}>\n"
-      result << "  <dn>#{dn}</dn>\n"
-      normalize_data(@data).sort_by {|key, values| key}.each do |key, values|
-        targets = []
-        values.each do |value|
-          if value.is_a?(Hash)
-            value.each do |option, real_value|
-              targets << [real_value, " #{option}=\"true\""]
-            end
+      options = options.dup
+      options[:root] ||= (self.class.name || '').underscore
+      except = options[:except]
+      if except
+        options[:except] = except.collect do |name|
+          if name.to_s.downcase == "dn"
+            "dn"
           else
-            targets << [value]
+            to_real_attribute_name(name)
           end
-        end
-        targets.sort_by {|value, attr| value}.each do |value, attr|
-          result << "  <#{key}#{attr}>#{value}</#{key}>\n"
-        end
+        end.compact
       end
-      result << "</#{root}>\n"
-      result
+      XML.new(dn, normalize_data(@data), schema).to_s(options)
+    end
+
+    def to_s
+      to_ldif
     end
 
     def have_attribute?(name, except=[])
       real_name = to_real_attribute_name(name)
-      real_name and !except.include?(real_name)
+      !real_name.nil? and !except.include?(real_name)
     end
     alias_method :has_attribute?, :have_attribute?
 
@@ -888,7 +963,7 @@ module ActiveLdap
       end
       config = {:bind_dn => dn, :allow_anonymous => false}.merge(config)
       config[:password_block] ||= block if block_given?
-      establish_connection(config)
+      setup_connection(config)
 
       before_connection = @connection
       begin
@@ -922,11 +997,14 @@ module ActiveLdap
 
     alias_method :base_of_class, :base
     def base
-      [@base, base_of_class].compact.join(",")
+      ensure_update_dn
+      [@base, base_of_class].find_all do |component|
+        not component.blank?
+      end.join(",")
     end
 
-    undef_method :base=
     def base=(object_local_base)
+      ensure_update_dn
       @dn = nil
       @base = object_local_base
     end
@@ -936,7 +1014,6 @@ module ActiveLdap
       @scope || scope_of_class
     end
 
-    undef_method :scope=
     def scope=(scope)
       self.class.validate_scope(scope)
       @scope = scope
@@ -959,6 +1036,17 @@ module ActiveLdap
     end
 
     private
+    def dn_attribute_with_fallback
+      begin
+        dn_attribute
+      rescue DistinguishedNameInvalid
+        _dn_attribute = @dn_attribute || dn_attribute_of_class
+        _dn_attribute = to_real_attribute_name(_dn_attribute) || _dn_attribute
+        raise if _dn_attribute.nil?
+        _dn_attribute
+      end
+    end
+
     def inspect_attribute(name)
       values = get_attribute(name, true)
       values.collect do |value|
@@ -1005,6 +1093,8 @@ module ActiveLdap
 
     def initialize_by_ldap_data(dn, attributes)
       init_base
+      dn = Compatible.convert_to_utf8_encoded_object(dn)
+      attributes = Compatible.convert_to_utf8_encoded_object(attributes)
       @dn = dn
       @new_entry = false
       @dn_is_base = false
@@ -1014,7 +1104,6 @@ module ActiveLdap
       self.dn = dn
       self.attributes = attributes
       yield self if block_given?
-      assert_dn_attribute
     end
 
     def instantiate(args)
@@ -1029,7 +1118,7 @@ module ActiveLdap
       obj
     end
 
-    def to_real_attribute_name(name, allow_normalized_name=false)
+    def to_real_attribute_name(name, allow_normalized_name=true)
       return name if name.nil?
       if allow_normalized_name
         entry_attribute.normalize(name, allow_normalized_name) ||
@@ -1059,6 +1148,8 @@ module ActiveLdap
       @base = nil
       @scope = nil
       @dn = nil
+      @dn_is_base = false
+      @dn_split_value = nil
       @connection ||= nil
       clear_connection_based_cache
     end
@@ -1119,35 +1210,61 @@ module ActiveLdap
     #
     # Set the value of the attribute called by method_missing?
     def set_attribute(name, value)
-      attr = to_real_attribute_name(name)
-      attr, value = update_dn(attr, value) if attr == dn_attribute
-      raise UnknownAttribute.new(name) if attr.nil?
+      real_name = to_real_attribute_name(name)
+      _dn_attribute = nil
+      valid_dn_attribute = true
+      begin
+        _dn_attribute = dn_attribute
+      rescue DistinguishedNameInvalid
+        valid_dn_attribute = false
+      end
+      if valid_dn_attribute and real_name == _dn_attribute
+        real_name, value = register_new_dn_attribute(real_name, value)
+      end
+      raise UnknownAttribute.new(name) if real_name.nil?
 
-      @data[attr] = value
+      @data[real_name] = value
     end
 
-    def update_dn(attr, value)
+    def register_new_dn_attribute(name, value)
       @dn = nil
       @dn_is_base = false
-      return [attr, nil] if value.blank?
+      if value.blank?
+        @dn_split_value = nil
+        [name, nil]
+      else
+        new_name, new_value, raw_new_value, new_bases = split_dn_value(value)
+        @dn_split_value = [new_name, new_value, new_bases]
+        if new_name.nil? and new_value.nil?
+          new_name, raw_new_value = new_bases[0].to_a[0]
+        end
+        [to_real_attribute_name(new_name) || name,
+         raw_new_value || value]
+      end
+    end
 
-      new_dn_attribute, new_value, bases = split_dn_value(value)
-      if new_dn_attribute.nil? and new_value.nil?
+    def update_dn(new_name, new_value, bases)
+      if new_name.nil? and new_value.nil?
         @dn_is_base = true
         @base = nil
         attr, value = bases[0].to_a[0]
         @dn_attribute = attr
       else
-        new_dn_attribute = to_real_attribute_name(new_dn_attribute)
-        if new_dn_attribute
-          value = new_value
-          @base = bases.empty? ? nil : DN.new(*bases).to_s
-          if dn_attribute != new_dn_attribute
-            @dn_attribute = attr = new_dn_attribute
-          end
+        new_name ||= @dn_attribute || dn_attribute_of_class
+        new_name = to_real_attribute_name(new_name)
+        if new_name.nil?
+          new_name = @dn_attribute || dn_attribute_of_class
+          new_name = to_real_attribute_name(new_name)
         end
+        new_bases = bases.empty? ? nil : DN.new(*bases).to_s
+        dn_components = ["#{new_name}=#{new_value}",
+                         new_bases,
+                         base_of_class]
+        dn_components = dn_components.find_all {|component| !component.blank?}
+        DN.parse(dn_components.join(','))
+        @base = new_bases
+        @dn_attribute = new_name
       end
-      [attr, value]
     end
 
     def split_dn_value(value)
@@ -1156,9 +1273,14 @@ module ActiveLdap
         dn_value = value if value.is_a?(DN)
         dn_value ||= DN.parse(value)
       rescue DistinguishedNameInvalid
-        dn_value = DN.parse("#{dn_attribute}=#{value}")
+        begin
+          dn_value = DN.parse("#{dn_attribute}=#{value}")
+        rescue DistinguishedNameInvalid
+          return [nil, value, value, []]
+        end
       end
 
+      val = bases = nil
       begin
         relative_dn_value = dn_value - self.class.parsed_base
         if relative_dn_value.rdns.empty?
@@ -1172,20 +1294,41 @@ module ActiveLdap
       end
 
       dn_attribute_name, dn_attribute_value = val.to_a[0]
-      [dn_attribute_name, dn_attribute_value, bases]
+      escaped_dn_attribute_value = nil
+      unless dn_attribute_value.nil?
+        escaped_dn_attribute_value = DN.escape_value(dn_attribute_value)
+      end
+      [dn_attribute_name, escaped_dn_attribute_value,
+       dn_attribute_value, bases]
+    end
+
+    def need_update_dn?
+      not @dn_split_value.nil?
+    end
+
+    def ensure_update_dn
+      return unless need_update_dn?
+      @mutex.synchronize do
+        if @dn_split_value
+          update_dn(*@dn_split_value)
+          @dn_split_value = nil
+        end
+      end
     end
 
     def compute_dn(escape_dn_value=false)
       return base if @dn_is_base
 
+      ensure_update_dn
       dn_value = id
       if dn_value.nil?
-        raise DistinguishedNameNotSetError.new,
-                _("%s's DN attribute (%s) isn't set") % [self, dn_attribute]
+        format =_("%s's DN attribute (%s) isn't set")
+        message = format % [self.inspect, dn_attribute]
+        raise DistinguishedNameNotSetError.new, message
       end
-      dn_value = DN.escape_value(dn_value) if escape_dn_value
+      dn_value = DN.escape_value(dn_value.to_s) if escape_dn_value
       _base = base
-      _base = nil if _base.empty?
+      _base = nil if _base.blank?
       ["#{dn_attribute}=#{dn_value}", _base].compact.join(",")
     end
 
@@ -1247,8 +1390,12 @@ module ActiveLdap
 
         next if v == value
 
+        x = value
+        value = self.class.remove_blank_value(value) || []
+        next if v == value
+
         # Create mod entries
-        if value.empty?
+        if self.class.blank_value?(value)
           # Since some types do not have equality matching rules,
           # delete doesn't work
           # Replacing with nothing is equivalent.
@@ -1263,7 +1410,11 @@ module ActiveLdap
       end
       data.each do |k, v|
         value = v || []
-        next if ldap_data.has_key?(k) or value.empty?
+        next if ldap_data.has_key?(k)
+
+        value = self.class.remove_blank_value(value) || []
+        next if self.class.blank_value?(value)
+
 
         # Detect subtypes and account for them
         # REPLACE will function like ADD, but doesn't hit EQUALITY problems
@@ -1284,7 +1435,9 @@ module ActiveLdap
       oc_value = data['objectClass']
       attributes.push(['objectClass', oc_value])
       data.each do |key, value|
-        next if value.empty? or key == 'objectClass' or key == dn_attr
+        next if key == 'objectClass' or key == dn_attr
+        value = self.class.remove_blank_value(value)
+        next if self.class.blank_value?(value)
 
         attributes.push([key, value])
       end
@@ -1292,13 +1445,6 @@ module ActiveLdap
       attributes
     end
 
-    def assert_dn_attribute
-      unless dn_attribute
-        raise ConfigurationError,
-                _("dn_attribute isn't set for this class: %s") % self.class
-      end
-    end
-
     def create_or_update
       new_entry? ? create : update
     end
