@@ -23,7 +23,7 @@ module ActiveLdap
 
     DEFAULT_CONFIG = {}
     DEFAULT_CONFIG[:host] = '127.0.0.1'
-    DEFAULT_CONFIG[:port] = 389
+    DEFAULT_CONFIG[:port] = nil
     DEFAULT_CONFIG[:method] = :plain  # :ssl, :tls, :plain allowed
 
     DEFAULT_CONFIG[:bind_dn] = nil
@@ -31,14 +31,14 @@ module ActiveLdap
     DEFAULT_CONFIG[:password] = nil
     DEFAULT_CONFIG[:store_password] = true
     DEFAULT_CONFIG[:allow_anonymous] = true
-    DEFAULT_CONFIG[:sasl_quiet] = false
+    DEFAULT_CONFIG[:sasl_quiet] = true
     DEFAULT_CONFIG[:try_sasl] = false
     # See http://www.iana.org/assignments/sasl-mechanisms
     DEFAULT_CONFIG[:sasl_mechanisms] = ["GSSAPI", "DIGEST-MD5",
                                         "CRAM-MD5", "EXTERNAL"]
 
-    DEFAULT_CONFIG[:retry_limit] = 3
-    DEFAULT_CONFIG[:retry_wait] = 3
+    DEFAULT_CONFIG[:retry_limit] = 1
+    DEFAULT_CONFIG[:retry_wait] = 1
     DEFAULT_CONFIG[:timeout] = 0 # in seconds; 0 <= Never timeout
     # Whether or not to retry on timeouts
     DEFAULT_CONFIG[:retry_on_timeout] = true
