@@ -1,7 +1,8 @@
-begin
-  require "gettext/active_record"
+if Object.const_defined?(:GetText)
   ActiveLdap.const_set("GetText", GetText)
-rescue LoadError
+end
+
+unless ActiveLdap.const_defined?(:GetText)
   require 'active_ldap/get_text_fallback'
 end
 
