@@ -34,7 +34,11 @@ module ActiveLdap
 
         value = value.first if value.is_a?(Array) and value.first.size == 1
 
-        _attr, value, _prefix = split_search_value(value)
+        _attr = nil
+        _prefix = nil
+        if attr.nil? or attr == dn_attribute
+          _attr, value, _prefix = split_search_value(value)
+        end
         attr ||= _attr || ensure_search_attribute
         prefix ||= _prefix
         filter ||= [attr, value]
@@ -58,18 +62,18 @@ module ActiveLdap
         }
 
         options[:connection] ||= connection
-        options[:connection].search(search_options) do |dn, attrs|
+        values = options[:connection].search(search_options) do |dn, attrs|
           attributes = {}
-          attrs.each do |key, value|
+          attrs.each do |key, _value|
             normalized_attr, normalized_value =
-              normalize_attribute_options(key, value)
+              normalize_attribute_options(key, _value)
             attributes[normalized_attr] ||= []
             attributes[normalized_attr].concat(normalized_value)
           end
-          value = [dn, attributes]
-          value = yield(value) if block_given?
-          value
+          [dn, attributes]
         end
+        values = values.collect {|_value| yield(_value)} if block_given?
+        values
       end
 
       def exist?(dn, options={})
@@ -117,7 +121,9 @@ module ActiveLdap
       end
 
       def ensure_base(target)
-        [truncate_base(target),  base].join(',')
+        [truncate_base(target), base].reject do |component|
+          component.blank?
+        end.join(',')
       end
 
       def truncate_base(target)
@@ -190,14 +196,18 @@ module ActiveLdap
       #
       # Finds the first match for value where |value| is the value of some
       # |field|, or the wildcard match. This is only useful for derived classes.
-      # usage: Subclass.find(:attribute => "cn", :value => "some*val")
-      #        Subclass.find('some*val')
+      # usage: Subclass.find(:all, :attribute => "cn", :value => "some*val")
+      #        Subclass.find(:all, 'some*val')
       def find(*args)
         options = extract_options_from_args!(args)
         args = [:first] if args.empty? and !options.empty?
         case args.first
         when :first
+          options[:value] ||= args[1]
           find_initial(options)
+        when :last
+          options[:value] ||= args[1]
+          find_last(options)
         when :all
           options[:value] ||= args[1]
           find_every(options)
@@ -206,11 +216,38 @@ module ActiveLdap
         end
       end
 
+      # A convenience wrapper for <tt>find(:first,
+      # *args)</tt>. You can pass in all the same arguments
+      # to this method as you can to <tt>find(:first)</tt>.
+      def first(*args)
+        find(:first, *args)
+      end
+
+      # A convenience wrapper for <tt>find(:last,
+      # *args)</tt>. You can pass in all the same arguments
+      # to this method as you can to <tt>find(:last)</tt>.
+      def last(*args)
+        find(:last, *args)
+      end
+
+      # This is an alias for find(:all).  You can pass in
+      # all the same arguments to this method as you can
+      # to find(:all)
+      def all(*args)
+        find(:all, *args)
+      end
+
       private
       def find_initial(options)
         find_every(options.merge(:limit => 1)).first
       end
 
+      def find_last(options)
+        order = options[:order] || self.order || 'ascend'
+        order = normalize_sort_order(order) == :ascend ? :descend : :ascend
+        find_initial(options.merge(:order => order))
+      end
+
       def normalize_sort_order(value)
         case value.to_s
         when /\Aasc(?:end)?\z/i
@@ -383,10 +420,12 @@ module ActiveLdap
 
       module ModifyRecordLoadable
         def load(operator, options)
-          each do |operation|
-            operator.modify_entry(dn, operation.to_modify_entries,
-                                  {:controls => controls}.merge(options))
+          modify_entries = operations.inject([]) do |result, operation|
+            result + operation.to_modify_entries
           end
+          return if modify_entries.empty?
+          operator.modify_entry(dn, modify_entries,
+                                {:controls => controls}.merge(options))
         end
 
         module AddOperationModifiable
@@ -501,9 +540,9 @@ module ActiveLdap
         if dn.is_a?(Array)
           i = -1
           dns = dn
-          dns.collect do |dn|
+          dns.collect do |_dn|
             i += 1
-            update(dn, attributes[i], options)
+            update(_dn, attributes[i], options)
           end
         else
           object = find(dn, options)
