@@ -1,7 +1,9 @@
 module ActiveLdap
   class Schema
+    include GetTextSupport
+
     def initialize(entries)
-      @entries = default_entries.merge(entries || {})
+      @entries = normalize_entries(entries || {})
       @schema_info = {}
       @class_attributes_info = {}
       @cache = {}
@@ -123,6 +125,12 @@ module ActiveLdap
       end
     end
 
+    def dit_content_rule_attribute(name, attribute_name)
+      cache([:dit_content_rule_attribute, name, attribute_name]) do
+        fetch("dITContentRules", name, attribute_name)
+      end
+    end
+
     def ldap_syntax(name)
       cache([:ldap_syntax, name]) do
         Syntax.new(name, self)
@@ -143,6 +151,17 @@ module ActiveLdap
       end
     end
 
+    def dump(output=nil)
+      require 'pp'
+      output ||= STDOUT
+      if output.respond_to?(:write)
+        PP.pp(@entries, output)
+      else
+        open(output, "w") {|out| PP.pp(@entries, out)}
+      end
+      nil
+    end
+
     private
     def cache(key)
       (@cache[key] ||= [yield])[0]
@@ -189,7 +208,7 @@ module ActiveLdap
 
     def parse_attributes(str, attributes)
       str.scan(/([A-Z\-_]+)\s+
-                (?:\(\s*([\w\-]+(?:\s+\$\s+[\w\-]+)+)\s*\)|
+                (?:\(\s*(\w[\w\-;]*(?:\s+\$\s+\w[\w\-;]*)*)\s*\)|
                    \(\s*([^\)]*)\s*\)|
                    '([^\']*)'|
                    ((?!#{RESERVED_NAMES_RE})[a-zA-Z][a-zA-Z\d\-;]*)|
@@ -242,9 +261,23 @@ module ActiveLdap
         "objectClasses" => [],
         "attributeTypes" => [],
         "ldapSyntaxes" => [],
+        "dITContentRules" => [],
+        "matchingRules" => [],
       }
     end
 
+    def normalize_entries(entries)
+      normalized_entries = default_entries
+      normalized_keys = normalized_entries.keys
+      entries.each do |name, values|
+        normalized_name = normalized_keys.find do |key|
+          key.downcase == name
+        end
+        normalized_entries[normalized_name || name] = values
+      end
+      normalized_entries
+    end
+
     class Entry
       include Comparable
 
@@ -260,11 +293,12 @@ module ActiveLdap
 
       def eql?(other)
         self.class == other.class and
-          id == other.id
+          (id == other.id or
+           (id.nil? and other.nil? and name == other.name))
       end
 
       def hash
-        id.hash
+        id.nil? ? name.hash : id.hash
       end
 
       def <=>(other)
@@ -611,9 +645,13 @@ module ActiveLdap
         end
       end
 
+      UNWRITABLE_MUST_ATTRIBUTES = ["nTSecurityDescriptor"]
       def collect_attributes
-        must = attribute('MUST').collect {|name| @schema.attribute(name)}
-        may = attribute('MAY').collect {|name| @schema.attribute(name)}
+        must = attribute('MUST').reject do |name|
+          UNWRITABLE_MUST_ATTRIBUTES.include?(name)
+        end.uniq
+        must = must.collect {|name| @schema.attribute(name)}
+        may = attribute('MAY').uniq.collect {|name| @schema.attribute(name)}
 
         all_must = must.dup
         all_may = may.dup
@@ -630,7 +668,8 @@ module ActiveLdap
       end
 
       def attribute(attribute_name, name=@name)
-        @schema.object_class_attribute(name, attribute_name)
+        @schema.object_class_attribute(name, attribute_name) +
+          @schema.dit_content_rule_attribute(name, attribute_name)
       end
     end
   end
