@@ -4,6 +4,47 @@ class TestAssociations < Test::Unit::TestCase
   include AlTestUtils
 
   priority :must
+  def test_belongs_to_with_invalid_dn_attribute_value
+    make_temporary_user do |user,|
+      make_temporary_group do |group|
+        user.primary_group = group
+        user.uid = "#"
+        assert_nothing_raised do
+          user.primary_group.reload
+        end
+      end
+    end
+  end
+
+  priority :normal
+  def test_has_many_wrap_with_nonexistent_entry
+    @user_class.has_many :references, :wrap => "seeAlso", :primary_key => "dn"
+    @user_class.set_associated_class(:references, @group_class)
+    @group_class.belongs_to :related_users, :many => "seeAlso",
+                            :foreign_key => "dn"
+    @group_class.set_associated_class(:related_users, @user_class)
+    make_temporary_user do |user,|
+      make_temporary_group do |group1|
+        make_temporary_group do |group2|
+          user.references = [group1, group2]
+          group3_dn = group2.dn.sub(/cn=(.*?),/, "cn=\\1-nonexistent,")
+          user.see_also += [group3_dn]
+          user.save!
+
+          user = @user_class.find(user.dn)
+          assert_equal([group1.dn, group2.dn, group3_dn],
+                       user.see_also.collect(&:to_s))
+          assert_equal([group1.dn, group2.dn, group3_dn],
+                       user.references.collect(&:dn))
+          assert_equal([group1.gid_number, group2.gid_number, nil],
+                       user.references.collect(&:gid_number))
+          assert_equal([false, false, true],
+                       user.references.collect(&:new_entry?))
+        end
+      end
+    end
+  end
+
   def test_has_many_wrap_with_dn_value
     @user_class.has_many :references, :wrap => "seeAlso", :primary_key => "dn"
     @user_class.set_associated_class(:references, @group_class)
@@ -42,7 +83,6 @@ class TestAssociations < Test::Unit::TestCase
     end
   end
 
-  priority :normal
   def test_belongs_to_many_with_dn_value
     @user_class.has_many :references, :wrap => "seeAlso", :primary_key => "dn"
     @user_class.set_associated_class(:references, @group_class)
@@ -185,12 +225,12 @@ class TestAssociations < Test::Unit::TestCase
     mod = Module.new
     mod.__send__(:mattr_accessor, :called)
     mod.__send__(:define_method, :replace) do |entries|
-      super
+      super(entries)
       mod.called = true
     end
     mod.called = false
 
-    @group_class.send(:undef_method, :members, :members=)
+    @group_class.send(:undef_method, :members, :members=, :__make_members)
     @group_class.has_many :members, :wrap => "memberUid",
                           :extend => mod
     @group_class.set_associated_class(:members, @user_class)
@@ -230,7 +270,7 @@ class TestAssociations < Test::Unit::TestCase
           group.members = [user1]
           assert_equal([user1.uid].sort,
                        group.members.collect {|x| x.uid}.sort)
-          assert_equal([user1.uid].sort, group.member_uid.sort)
+          assert_equal(user1.uid, group.member_uid)
           assert_equal(gid_number1, user1.gid_number.to_i)
         end
       end
@@ -243,14 +283,15 @@ class TestAssociations < Test::Unit::TestCase
                              :scope => :sub,
                              :classes => ["posixGroup"]
     assert_raises(ArgumentError) do
-      group_class.has_many :members, :class_name => "User"
+      group_class.has_many :users, :unknown_name => "value"
     end
 
     mod = Module.new
     assert_nothing_raised do
-      group_class.has_many :members, :class => "User", :wrap => "memberUid",
+      group_class.has_many :users, :class_name => "User"
+      group_class.has_many :members, :class => @user_class, :wrap => "memberUid",
                            :extend => mod
-      group_class.has_many :primary_members, :class => "User",
+      group_class.has_many :primary_members, :class => @user_class,
                            :foreign_key => "gidNumber",
                            :primary_key => "gidNumber",
                            :extend => mod
@@ -264,14 +305,16 @@ class TestAssociations < Test::Unit::TestCase
                             :scope => :sub,
                             :classes => ["posixAccount", "person"]
     assert_raises(ArgumentError) do
-      user_class.belongs_to :groups, :class_name => "Group"
+      user_class.belongs_to :groups, :unknown_name => "value"
     end
 
     mod = Module.new
     assert_nothing_raised do
-      user_class.belongs_to :groups, :class => "Group", :many => "memberUid",
+      user_class.belongs_to :string_groups, :class_name => "Group"
+      user_class.belongs_to :groups, :class => @group_class,
+                            :many => "memberUid",
                             :extend => mod
-      user_class.belongs_to :primary_group, :class => "Group",
+      user_class.belongs_to :primary_group, :class => @group_class,
                             :foreign_key => "gidNumber",
                             :primary_key => "gidNumber",
                             :extend => mod
@@ -409,7 +452,7 @@ class TestAssociations < Test::Unit::TestCase
           group.members << user1
           assert_equal([user1.uid].sort,
                        group.members.collect {|x| x.uid}.sort)
-          assert_equal([user1.uid].sort, group.member_uid.sort)
+          assert_equal(user1.uid, group.member_uid)
           assert_equal(gid_number1, user1.gid_number.to_i)
 
           assert_equal(gid_number2, user2.gid_number.to_i)
