@@ -6,8 +6,127 @@ class TestBase < Test::Unit::TestCase
   include AlTestUtils
 
   priority :must
+  def test_first
+    make_temporary_user(:simple => true) do |user1,|
+      make_temporary_user(:simple => true) do |user2,|
+        assert_equal(user1, @user_class.find(:first))
+        assert_equal(user2, @user_class.find(:first, user2.cn))
+      end
+    end
+  end
+
+  def test_last
+    make_temporary_user(:simple => true) do |user1,|
+      make_temporary_user(:simple => true) do |user2,|
+        assert_equal(user2, @user_class.find(:last))
+        assert_equal(user1, @user_class.find(:last, user1.cn))
+      end
+    end
+  end
+
+  def test_convenient_operation_methods
+    make_temporary_user(:simple => true) do |user1,|
+      make_temporary_user(:simple => true) do |user2,|
+        assert_equal(user1, @user_class.first)
+        assert_equal(user2, @user_class.last)
+        assert_equal([user1, user2], @user_class.all)
+      end
+    end
+  end
+
+  priority :normal
+  def test_set_attributes_with_a_blank_value_in_values
+    make_temporary_user(:simple => true) do |user,|
+      user.attributes = {"description" => ["a", "b", ""]}
+      assert(user.save)
+    end
+  end
+
+  def test_set_attributes_with_a_blank_value
+    make_temporary_user(:simple => true) do |user,|
+      user.attributes = {"description" => [""]}
+      assert(user.save)
+    end
+  end
+
+  def test_create_invalid
+    user = @user_class.create
+    assert_not_predicate(user.errors, :empty?)
+  end
+
+  def test_id_with_invalid_dn_attribute_value
+    user = @user_class.new("#")
+    assert_equal("#", user.uid)
+    assert_equal("#", user.id)
+  end
+
+  def test_non_string_dn_attribute_value
+    user = @user_class.new("uidNumber=10110")
+    user.uid = user.cn = user.sn = "test-user"
+    user.gid_number = 10000
+    user.home_directory = "/home/test-user"
+    assert_nothing_raised do
+      user.save!
+    end
+  end
+
+  def test_set_dn_with_unnormalized_dn_attribute
+    make_temporary_user do |user,|
+      assert_not_equal("ZZZ", user.cn)
+      user.dn = "CN=ZZZ"
+      assert_equal("ZZZ", user.cn)
+    end
+  end
+
+  def test_destroy_with_empty_base_and_prefix_of_class
+    make_temporary_user do |user,|
+      base = user.class.base
+      prefix = user.class.prefix
+      begin
+        user.class.base = ""
+        user.class.prefix = ""
+        user.base = base
+        user.destroy
+      ensure
+        user.class.base = base
+        user.class.prefix = prefix
+      end
+    end
+  end
+
+  def test_empty_base_of_class
+    make_temporary_user do |user,|
+      user.class.prefix = ""
+      user.class.base = ""
+      user.base = "dc=net"
+      assert_equal("dc=net", user.base)
+    end
+  end
+
+  def test_search_value_with_no_dn_attribute
+    make_temporary_user do |user1,|
+      make_temporary_user do |user2,|
+        options = {:attribute => "seeAlso", :value => user2.dn}
+        assert_equal([],
+                     user1.class.find(:all, options).collect(&:dn))
+
+        user1.see_also = user2.dn
+        user1.save!
+
+        assert_equal([user1.dn],
+                     user1.class.find(:all, options).collect(&:dn))
+      end
+    end
+  end
+
+  def test_to_s
+    make_temporary_group do |group|
+      assert_equal(group.to_s, group.to_ldif)
+    end
+  end
+
   def test_to_ldif
-    make_temporary_group do |group,|
+    make_temporary_group do |group|
       assert_to_ldif(group)
 
       group.gidNumber += 1
@@ -16,7 +135,6 @@ class TestBase < Test::Unit::TestCase
     end
   end
 
-  priority :normal
   def test_save_with_changes
     make_temporary_user do |user, password|
       user.cn += "!!!"
@@ -528,38 +646,143 @@ class TestBase < Test::Unit::TestCase
     assert_equal(<<-EOX, ou.to_xml(:root => "ou"))
 <ou>
   <dn>#{ou.dn}</dn>
-  <objectClass>organizationalUnit</objectClass>
-  <objectClass>top</objectClass>
-  <ou>Sample</ou>
+  <objectClasses type="array">
+    <objectClass>organizationalUnit</objectClass>
+    <objectClass>top</objectClass>
+  </objectClasses>
+  <ous type="array">
+    <ou>Sample</ou>
+  </ous>
 </ou>
 EOX
 
     assert_equal(<<-EOX, ou.to_xml)
 <>
   <dn>#{ou.dn}</dn>
-  <objectClass>organizationalUnit</objectClass>
-  <objectClass>top</objectClass>
-  <ou>Sample</ou>
+  <objectClasses type="array">
+    <objectClass>organizationalUnit</objectClass>
+    <objectClass>top</objectClass>
+  </objectClasses>
+  <ous type="array">
+    <ou>Sample</ou>
+  </ous>
 </>
 EOX
 
     make_temporary_user do |user, password|
-    assert_equal(<<-EOX, user.to_xml(:root => "user"))
+      xml = normalize_attributes_order(user.to_xml(:root => "user"))
+      assert_equal(<<-EOX, xml)
+<user>
+  <dn>#{user.dn}</dn>
+  <cns type="array">
+    <cn>#{user.cn}</cn>
+  </cns>
+  <gidNumber>#{user.gid_number}</gidNumber>
+  <homeDirectory>#{user.home_directory}</homeDirectory>
+  <jpegPhotos type="array">
+    <jpegPhoto base64="true">#{base64(jpeg_photo)}</jpegPhoto>
+  </jpegPhotos>
+  <objectClasses type="array">
+    <objectClass>inetOrgPerson</objectClass>
+    <objectClass>organizationalPerson</objectClass>
+    <objectClass>person</objectClass>
+    <objectClass>posixAccount</objectClass>
+    <objectClass>shadowAccount</objectClass>
+  </objectClasses>
+  <sns type="array">
+    <sn>#{user.sn}</sn>
+  </sns>
+  <uids type="array">
+    <uid>#{user.uid}</uid>
+  </uids>
+  <uidNumber>#{user.uid_number}</uidNumber>
+  <userCertificates type="array">
+    <userCertificate base64="true" binary="true">#{base64(certificate)}</userCertificate>
+  </userCertificates>
+  <userPasswords type="array">
+    <userPassword>#{user.user_password}</userPassword>
+  </userPasswords>
+</user>
+EOX
+    end
+  end
+
+  def test_to_xml_except
+    ou = ou_class.new("Sample")
+    assert_equal(<<-EOX, ou.to_xml(:root => "sample", :except => [:objectClass]))
+<sample>
+  <dn>#{ou.dn}</dn>
+  <ous type="array">
+    <ou>Sample</ou>
+  </ous>
+</sample>
+EOX
+
+    except = [:dn, :object_class]
+    assert_equal(<<-EOX, ou.to_xml(:root => "sample", :except => except))
+<sample>
+  <ous type="array">
+    <ou>Sample</ou>
+  </ous>
+</sample>
+EOX
+  end
+
+  def test_to_xml_escape
+    make_temporary_user do |user, password|
+      sn = user.sn
+      user.sn = "<#{sn}>"
+      except = [:jpeg_photo, :user_certificate]
+      assert_equal(<<-EOX, user.to_xml(:root => "user", :except => except))
+<user>
+  <dn>#{user.dn}</dn>
+  <cns type="array">
+    <cn>#{user.cn}</cn>
+  </cns>
+  <gidNumber>#{user.gid_number}</gidNumber>
+  <homeDirectory>#{user.home_directory}</homeDirectory>
+  <objectClasses type="array">
+    <objectClass>inetOrgPerson</objectClass>
+    <objectClass>organizationalPerson</objectClass>
+    <objectClass>person</objectClass>
+    <objectClass>posixAccount</objectClass>
+    <objectClass>shadowAccount</objectClass>
+  </objectClasses>
+  <sns type="array">
+    <sn>&lt;#{sn}&gt;</sn>
+  </sns>
+  <uids type="array">
+    <uid>#{user.uid}</uid>
+  </uids>
+  <uidNumber>#{user.uid_number}</uidNumber>
+  <userPasswords type="array">
+    <userPassword>#{user.user_password}</userPassword>
+  </userPasswords>
+</user>
+EOX
+    end
+  end
+
+  def test_to_xml_type_ldif
+    make_temporary_user do |user, password|
+      sn = user.sn
+      user.sn = "<#{sn}>"
+      except = [:jpeg_photo, :user_certificate]
+      options = {:root => "user", :except => except, :type => :ldif}
+      assert_equal(<<-EOX, user.to_xml(options))
 <user>
   <dn>#{user.dn}</dn>
   <cn>#{user.cn}</cn>
   <gidNumber>#{user.gid_number}</gidNumber>
   <homeDirectory>#{user.home_directory}</homeDirectory>
-  <jpegPhoto>#{jpeg_photo}</jpegPhoto>
   <objectClass>inetOrgPerson</objectClass>
   <objectClass>organizationalPerson</objectClass>
   <objectClass>person</objectClass>
   <objectClass>posixAccount</objectClass>
   <objectClass>shadowAccount</objectClass>
-  <sn>#{user.sn}</sn>
+  <sn>&lt;#{sn}&gt;</sn>
   <uid>#{user.uid}</uid>
   <uidNumber>#{user.uid_number}</uidNumber>
-  <userCertificate binary="true">#{certificate}</userCertificate>
   <userPassword>#{user.user_password}</userPassword>
 </user>
 EOX
@@ -583,13 +806,13 @@ EOX
 
   def test_have_attribute?
     make_temporary_user do |user, password|
-      assert(user.have_attribute?(:cn))
-      assert(user.have_attribute?(:commonName))
-      assert(user.have_attribute?(:common_name))
-      assert(!user.have_attribute?(:commonname))
-      assert(!user.have_attribute?(:COMMONNAME))
+      assert_true(user.have_attribute?(:cn))
+      assert_true(user.have_attribute?(:commonName))
+      assert_true(user.have_attribute?(:common_name))
+      assert_true(user.have_attribute?(:commonname))
+      assert_true(user.have_attribute?(:COMMONNAME))
 
-      assert(!user.have_attribute?(:unknown_attribute))
+      assert_false(user.have_attribute?(:unknown_attribute))
     end
   end
 
@@ -613,7 +836,7 @@ EOX
 
   def test_update_all
     make_temporary_user do |user, password|
-      make_temporary_user do |user2, password|
+      make_temporary_user do |user2, password2|
         user2_cn = user2.cn
         new_cn = "New #{user.cn}"
         @user_class.update_all({:cn => new_cn}, user.uid)
@@ -639,7 +862,7 @@ EOX
       new_user = @user_class.update(user.dn, {:cn => new_cn})
       assert_equal(new_cn, new_user.cn)
 
-      make_temporary_user do |user2, password|
+      make_temporary_user do |user2, password2|
         new_sns = ["New SN1", "New SN2"]
         new_cn2 = "New #{user2.cn}"
         new_user, new_user2 = @user_class.update([user.dn, user2.dn],
@@ -680,4 +903,23 @@ EOX
     end
     assert_equal([entry], parsed_entries)
   end
+
+  def base64(string)
+    [string].pack("m").gsub(/\n/u, "")
+  end
+
+  def normalize_attributes_order(xml)
+    xml.gsub(/<(\S+) (.+?)(\/?)>/) do |matched|
+      name = $1
+      attributes = $2
+      close_mark = $3
+      attributes = attributes.scan(/(\S+)="(.+?)"/)
+      normalized_attributes = attributes.sort_by do |key, value|
+        key
+      end.collect do |key, value|
+        "#{key}=\"#{value}\""
+      end.join(' ')
+      "<#{name} #{normalized_attributes}#{close_mark}>"
+    end
+  end
 end
