@@ -9,8 +9,94 @@ class TestLDIF < Test::Unit::TestCase
   include AlTestUtils::ExampleFile
 
   priority :must
+  def test_accept_empty_lines_after_content
+    ldif_source = <<-EOL
+version: 1
+dn: dc=devel,dc=example,dc=com
+dc: devel
+objectClass: top
+objectClass: dcObject
+objectClass: organization
+o: devel
+EOL
+
+    expected_ldif_to_s = <<-EOL
+version: 1
+dn: dc=devel,dc=example,dc=com
+dc: devel
+o: devel
+objectClass: dcObject
+objectClass: organization
+objectClass: top
+EOL
+
+
+    assert_ldif_to_s(expected_ldif_to_s, ldif_source)
+    assert_ldif_to_s(expected_ldif_to_s, ldif_source + "\n")
+    assert_ldif_to_s(expected_ldif_to_s, ldif_source + "\n\n")
+    assert_ldif_to_s(expected_ldif_to_s, ldif_source + ("\n" * 10))
+  end
+
+  def test_accept_comments_after_content
+    ldif_source = <<-EOL
+version: 1
+dn: dc=devel,dc=example,dc=com
+dc: devel
+objectClass: top
+objectClass: dcObject
+objectClass: organization
+o: devel
+
+# search result
+
+# numResponse: 1
+# numEntries: 1
+
+EOL
+
+    assert_ldif_to_s(<<-EOL, ldif_source)
+version: 1
+dn: dc=devel,dc=example,dc=com
+dc: devel
+o: devel
+objectClass: dcObject
+objectClass: organization
+objectClass: top
+EOL
+  end
 
   priority :normal
+  def test_comments_and_empty_lines_before_content
+    ldif_source = <<-EOL
+version: 1
+
+#
+# LDAPv3
+# base <dc=devel,dc=clear-code,dc=com> with scope subtree
+# filter: (objectclass=*)
+# requesting: ALL
+#
+
+# devel.example.com
+dn: dc=devel,dc=example,dc=com
+dc: devel
+objectClass: top
+objectClass: dcObject
+objectClass: organization
+o: devel
+EOL
+
+    assert_ldif_to_s(<<-EOL, ldif_source)
+version: 1
+dn: dc=devel,dc=example,dc=com
+dc: devel
+o: devel
+objectClass: dcObject
+objectClass: organization
+objectClass: top
+EOL
+  end
+
   def test_unknown_change_type
     ldif_source = <<-EOL
 version: 1
@@ -1126,6 +1212,7 @@ uid: hjensen
 telephonenumber: +1 408 555 1212
 jpegphoto:< file://#{jpeg_photo_path}
 EOL
+    set_encoding(ldif_source, "utf-8")
 
     jpeg_photo_attribute = "jpegphoto:: "
     value = [jpeg_photo].pack("m").gsub(/\n/, '')
@@ -1209,6 +1296,7 @@ sn;lang-en: Ogasawara
 cn;lang-en: Rodney Ogasawara
 title;lang-en: Sales, Director
 EOL
+    set_encoding(ldif_source, "utf-8")
 
     record1 = {
       "dn" => "ou=営業部,o=Airius",
@@ -1336,6 +1424,7 @@ sn;lang-en: Ogasawara
 cn;lang-en: Rodney Ogasawara
 title;lang-en: Sales, Director
 EOL
+    set_encoding(ldif_source, "utf-8")
 
     assert_ldif_to_s(<<-EOL, ldif_source)
 version: 1
@@ -1747,12 +1836,15 @@ EOL
 
   private
   def assert_ldif(version, records, ldif_source)
+    encoding = ldif_source.encoding if ldif_source.respond_to?(:encoding)
     ldif = ActiveLdap::Ldif.parse(ldif_source)
     assert_equal(version, ldif.version)
     assert_equal(records,
                  ldif.records.collect {|record| record.to_hash})
 
-    reparsed_ldif = ActiveLdap::Ldif.parse(ldif.to_s)
+    regenerated_ldif = ldif.to_s
+    set_encoding(regenerated_ldif, encoding)
+    reparsed_ldif = ActiveLdap::Ldif.parse(regenerated_ldif)
     assert_equal(ldif, reparsed_ldif)
 
     ldif
@@ -1783,4 +1875,8 @@ EOL
     ldif = ActiveLdap::Ldif.parse(original_ldif_source)
     assert_equal(expected_ldif_source, ldif.to_s)
   end
+
+  def set_encoding(string, encoding)
+    string.force_encoding("utf-8") if string.respond_to?(:force_encoding)
+  end
 end
