@@ -96,15 +96,13 @@ class TestReflection < Test::Unit::TestCase
     make_temporary_user do |user, password|
       attributes = user.must.collect(&:name) + user.may.collect(&:name)
       attributes -= ["objectClass"]
-      attributes = attributes.collect do |x|
-        Inflector.underscore(x)
-      end
+      attributes = attributes.collect(&:underscore)
       assert_equal([], attributes - user.methods)
       assert_equal([], attributes - user.methods(false))
 
       normalize_attributes_list = Proc.new do |*attributes_list|
         attributes_list.collect do |attrs|
-          attrs.collect {|x| Inflector.underscore(x)}
+          attrs.collect(&:underscore)
         end
       end
       assert_methods_with_only_required_classes(user, attributes,
@@ -115,15 +113,13 @@ class TestReflection < Test::Unit::TestCase
       user.remove_class("inetOrgPerson")
       attributes = user.must.collect(&:name) + user.may.collect(&:name)
       attributes -= ["objectClass"]
-      attributes = attributes.collect do |x|
-        Inflector.underscore(x)
-      end
+      attributes = attributes.collect(&:underscore)
       assert_equal([], attributes - user.methods)
       assert_equal([], attributes - user.methods(false))
 
       normalize_attributes_list = Proc.new do |*attributes_list|
         attributes_list.collect do |attrs|
-          attrs.collect {|x| Inflector.underscore(x)}
+          attrs.collect(&:underscore)
         end
       end
       assert_methods_with_only_required_classes(user, attributes,
