@@ -93,6 +93,26 @@ class TestSyntax < Test::Unit::TestCase
     assert_type_cast(Time.parse("1994/12/16 10:32:12.345 +09:00"),
                      "19941216103212.345+0900",
                      "Generalized Time")
+    begin
+      Time.utc(1601)
+      assert_type_cast(Time.utc(1601, 1, 1, 0, 4, 17),
+                       "16010101000417.0Z",
+                       "Generalized Time")
+    rescue ArgumentError
+      assert_type_cast(Time.at(0),
+                       "16010101000417.0Z",
+                       "Generalized Time")
+    end
+
+    begin
+      Time.at(-1)
+    rescue ArgumentError
+      if $!.message == "argument out of range"
+        assert_type_cast(Time.parse("1969/12/31 23:59:59 +00:00"),
+                         "19691231235959+0000",
+                         "Generalized Time")
+      end
+    end
   end
 
   def test_integer_type_cast
@@ -162,7 +182,12 @@ class TestSyntax < Test::Unit::TestCase
     assert_valid("これはDirectoryString文字列です。",
                  "Directory String")
 
-    value = NKF.nkf("-We", "これはDirectoryString文字列です。")
+    value = "これはDirectoryString文字列です。"
+    if value.respond_to?(:encode)
+      value = value.encode("euc-jp")
+    else
+      value = NKF.nkf("-We", value)
+    end
     assert_invalid(_("%s has invalid UTF-8 character") % value.inspect,
                    value, "Directory String")
   end
@@ -261,7 +286,12 @@ class TestSyntax < Test::Unit::TestCase
 
     assert_invalid(_("empty string"), "", "Postal Address")
 
-    value = NKF.nkf("-We", "東京")
+    value = "東京"
+    if value.respond_to?(:encode)
+      value = value.encode("euc-jp")
+    else
+      value = NKF.nkf("-We", value)
+    end
     assert_invalid(_("%s has invalid UTF-8 character") % value.inspect,
                    value, "Postal Address")
   end
