@@ -6,6 +6,62 @@ class TestValidation < Test::Unit::TestCase
   include ActiveLdap::Helper
 
   priority :must
+  def test_set_attributes_with_invalid_dn_attribute_value
+    user = nil
+    assert_nothing_raised do
+      user = @user_class.new(:uid => "=", :cn => "#")
+    end
+    assert(!user.valid?)
+  end
+
+  priority :normal
+  def test_set_attribute_to_invalid_dn_attribute_value_object
+    user = @user_class.new("=")
+    assert_nothing_raised do
+      user.uid_number = 11111
+    end
+    assert_equal(11111, user.uid_number)
+  end
+
+  def test_dn_validate_on_new
+    user = @user_class.new("=")
+    assert(!user.valid?)
+    reason = _("attribute value is missing")
+    invalid_format = _("%s is invalid distinguished name (DN): %s")
+    invalid_message = invalid_format % ["uid==,#{user.class.base}", reason]
+    format = _("%{fn} is invalid: %s")
+    format = format.sub(/^%\{fn\}/, la_('DN'))
+    message = format % invalid_message
+    assert_equal([message],
+                 user.errors.full_messages.find_all {|m| /DN/ =~ m})
+  end
+
+  def test_dn_validate
+    make_temporary_user do |user,|
+      user.uid = "="
+      assert(!user.valid?)
+      reason = _("attribute value is missing")
+      invalid_format = _("%s is invalid distinguished name (DN): %s")
+      invalid_message = invalid_format % ["uid==,#{user.class.base}", reason]
+      format = _("%{fn} is invalid: %s")
+      format = format.sub(/^%\{fn\}/, la_('DN'))
+      message = format % invalid_message
+      assert_equal([message], user.errors.full_messages)
+    end
+  end
+
+  def test_not_validate_empty_string
+    make_temporary_user do |user,|
+      assert(user.valid?)
+      user.uid_number = ""
+      assert(!user.valid?)
+      format = _("%{fn} is required attribute by objectClass '%s'")
+      format = format.sub(/^%\{fn\}/, la_('uidNumber'))
+      blank_message = format % loc_("posixAccount")
+      assert_equal([blank_message], user.errors.full_messages)
+    end
+  end
+
   def test_validate_excluded_classes
     make_temporary_user do |user,|
       user.save
@@ -13,19 +69,15 @@ class TestValidation < Test::Unit::TestCase
       assert(user.save)
       user.class.excluded_classes = ['person']
       assert(!user.save)
-      if ActiveLdap.get_text_supported?
-        format = n_("%{fn} has excluded value: %s",
-                    "%{fn} has excluded values: %s",
-                    1) % {:fn => la_("objectClass")}
-        message = format % loc_("person")
-      else
-        message = "objectClass has excluded value: person"
-      end
+      format = n_("%{fn} has excluded value: %s",
+                  "%{fn} has excluded values: %s",
+                  1)
+      format = format.sub(/^%\{fn\}/, la_("objectClass"))
+      message = format % loc_("person")
       assert_equal([message], user.errors.full_messages)
     end
   end
 
-  priority :normal
   def test_valid_subtype_and_single_value
     make_temporary_user do |user, password|
       user.display_name = [{"lang-ja" => ["ユーザ"]},
@@ -89,12 +141,10 @@ class TestValidation < Test::Unit::TestCase
     assert(ou_class.new("YYY").save)
     ou = ou_class.new("YYY")
     assert(!ou.save)
-    if ActiveLdap.get_text_supported?
-      format = _("%{fn} is duplicated: %s") % {:fn => la_("DN")}
-    else
-      format = "Dn " + _("is duplicated: %s")
-    end
-    assert_equal([format % ou.dn], ou.errors.full_messages)
+    format = _("%{fn} is duplicated: %s")
+    format = format.sub(/^%\{fn\}/, la_("DN"))
+    message = format % ou.dn
+    assert_equal([message], ou.errors.full_messages)
   end
 
   def test_save!
@@ -106,6 +156,9 @@ class TestValidation < Test::Unit::TestCase
       end
 
       @group_class.validates_presence_of(:description)
+      def @group_class.name
+        "Group"
+      end
       assert_raises(ActiveLdap::EntryInvalid) do
         group.save!
       end
@@ -121,6 +174,9 @@ class TestValidation < Test::Unit::TestCase
       assert_equal([], group.errors.to_a)
 
       @group_class.validates_presence_of(:description)
+      def @group_class.name
+        "Group"
+      end
       assert(!group.valid?)
       assert(group.errors.invalid?(:description))
       assert_equal(1, group.errors.size)
@@ -133,22 +189,18 @@ class TestValidation < Test::Unit::TestCase
     assert_not_nil(syntax_description)
     params = [formatted_value, syntax_description, reason]
     params.unshift(option) if option
-    if ActiveLdap.get_text_supported?
-      if option
-        format = _("%{fn} (%s) has invalid format: %s: required syntax: %s: %s")
-      else
-        format = _("%{fn} has invalid format: %s: required syntax: %s: %s")
-      end
-      format = format % {:fn => la_(name)}
-      assert_equal([format % params], model.errors.full_messages)
+    if option
+      format = _("%{fn}(%s) has invalid format: %s: required syntax: %s: %s")
     else
-      if option
-        format = _("(%s) has invalid format: %s: required syntax: %s: %s")
-      else
-        format = _("has invalid format: %s: required syntax: %s: %s")
-      end
-      assert_equal(["#{name} #{format % params}"], model.errors.full_messages)
+      format = _("%{fn} has invalid format: %s: required syntax: %s: %s")
     end
+    localized_name = la_(name)
+    if option and !ActiveLdap.get_text_supported?
+      localized_name += ' '
+    end
+    format = format.sub(/^%\{fn\}/, localized_name)
+    message = format % params
+    assert_equal([message], model.errors.full_messages)
   end
 
   def assert_invalid_see_also_value(invalid_value, value, option=nil)
